-- Uniquement pour mettre à jour les Esia qui étaient dans une ancienne version 3.4.0 (Ces Esia sont des Esia de test/non de prod).
ALTER TABLE IF EXISTS public.gesa_info  ADD COLUMN IF NOT EXISTS gesa_macadress macaddr;

CREATE TABLE IF NOT EXISTS esia_plugin_react_program
(
    react_plugin_id integer NOT NULL,
    react_program_id integer NOT NULL,
    react_parameter character varying(150) COLLATE pg_catalog."default",
    react_ok boolean,
    react_warning boolean,
    react_critical boolean,
    react_unknown boolean,
    CONSTRAINT "esia_react_PrimaryKey" PRIMARY KEY (react_plugin_id, react_program_id),
    CONSTRAINT "esia_react_ForeignKey_program_id" FOREIGN KEY (react_program_id)
        REFERENCES esia_program (program_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION,
    CONSTRAINT "esia_react_ForeignKey_plugin_id" FOREIGN KEY (react_plugin_id)
        REFERENCES esia_plugins (plugin_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
);

ALTER TABLE IF EXISTS esia_plugin_react_program OWNER to esia;


-- Dashboard : lien rapide
INSERT INTO public.dashboard_view(
	view_id, view_name, view_file_path, view_during, view_default_height, view_default_width, view_default_option, view_group_id)
	VALUES (DEFAULT, 'DASHBOARD_FAST_LINK', '/dashboard/view_link.php?id=:widgetid', 30, 100, 300, '{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"}}', NULL);


-- modfication pour ajouter la colone path

ALTER TABLE IF EXISTS public.esia_device_group  ADD COLUMN IF NOT EXISTS device_group_path text;

WITH RECURSIVE paths (device_group_id,name,path) AS (
	SELECT master.device_group_id, master.device_group_name, ARRAY[master.device_group_name]::character varying[] AS path
       FROM esia_device_group master
		WHERE device_group_id IN (SELECT device_group_id FROM esia_device_group WHERE device_group_parent IS NULL)
	UNION ALL
	SELECT sec.device_group_id, sec.device_group_name, paths.path || ARRAY[sec.device_group_name]
	 FROM esia_device_group sec
	 JOIN paths ON paths.device_group_id = sec.device_group_parent AND sec.device_group_name != ALL(paths.path)
)
UPDATE esia_device_group SET device_group_path=p.fullpath
FROM (select device_group_id,array_to_string(path,' > ')as fullpath from paths) as p
WHERE p.device_group_id = esia_device_group.device_group_id;


CREATE OR REPLACE FUNCTION public.gettopnodegroup(
	group_id integer)
    RETURNS esia_device_group
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE PARALLEL UNSAFE
AS $BODY$
	DECLARE
		v_row esia_device_group%ROWTYPE;
        BEGIN

		SELECT * INTO v_row
		FROM esia_device_group
		WHERE device_group_id = (SELECT device_group_parent FROM esia_device_group WHERE device_group_id=group_id);
		--Si le group_id est null c'est qu'on a demandé le top Group d'un group de top niveau (lvl1)
		IF v_row.device_group_id IS NULL
		THEN
			SELECT * INTO v_row FROM esia_device_group WHERE device_group_id=group_id;
			RETURN v_row;
		ELSE
			--Si le groupe parrent n'est pas null on est pas sur le top groupe
			IF v_row.device_group_parent IS NOT NULL
			THEN
				SELECT * INTO v_row  FROM getTopNodeGroup(v_row.device_group_id);
			END IF;
			RETURN v_row;
		END IF;


   END;
$BODY$;

ALTER FUNCTION public.gettopnodegroup(integer) OWNER TO esia;

CREATE OR REPLACE FUNCTION public.trigger_update_group_path()
    RETURNS trigger
    LANGUAGE 'plpgsql'
AS $BODY$
DECLARE
	v_var integer;
BEGIN
	--Si on est sur un top group
	v_var=0;
	IF NEW.device_group_parent IS NULL THEN
		UPDATE esia_device_group SET device_group_path = NEW.device_group_name WHERE device_group_id = NEW.device_group_id;
		v_var=1;
	END IF;
	IF v_var = 0 THEN
		--Si on est sur un top group
		WITH RECURSIVE paths (device_group_id,name,path) AS (
			SELECT master.device_group_id, master.device_group_name, ARRAY[master.device_group_path]::character varying[] AS path
       		FROM esia_device_group master
			WHERE device_group_id= (SELECT device_group_id FROM gettopnodegroup(NEW.device_group_id))
			UNION ALL
			SELECT sec.device_group_id, sec.device_group_name, paths.path || ARRAY[sec.device_group_name]
	 		FROM esia_device_group sec
	 		JOIN paths ON paths.device_group_id = sec.device_group_parent AND sec.device_group_name != ALL(paths.path)
		)
		UPDATE esia_device_group SET device_group_path=p.fullpath
		FROM (select device_group_id,array_to_string(path,' > ')as fullpath from paths) as p
		WHERE p.device_group_id = esia_device_group.device_group_id;
	END IF;
	IF TG_OP = 'UPDATE' THEN
		IF NEW.device_group_parent <> OLD.device_group_parent AND OLD.device_group_parent IS NOT NULL  THEN
		--Si on est sur un top group
		WITH RECURSIVE paths (device_group_id,name,path) AS (
			SELECT master.device_group_id, master.device_group_name, ARRAY[master.device_group_path]::character varying[] AS path
       		FROM esia_device_group master
			WHERE device_group_id= (SELECT device_group_id FROM gettopnodegroup(OLD.device_group_parent))
			UNION ALL
			SELECT sec.device_group_id, sec.device_group_name, paths.path || ARRAY[sec.device_group_name]
	 		FROM esia_device_group sec
	 		JOIN paths ON paths.device_group_id = sec.device_group_parent AND sec.device_group_name != ALL(paths.path)
		)
		UPDATE esia_device_group SET device_group_path=p.fullpath
		FROM (select device_group_id,array_to_string(path,' > ')as fullpath from paths) as p
		WHERE p.device_group_id = esia_device_group.device_group_id;
		END IF;
	END IF;
    RETURN NEW;
END;
$BODY$;
ALTER FUNCTION public.trigger_update_group_path() OWNER TO esia;
CREATE TRIGGER "updateGroupPath"
    AFTER INSERT OR UPDATE OF device_group_name, device_group_parent
    ON public.esia_device_group
    FOR EACH ROW
    EXECUTE FUNCTION public.trigger_update_group_path();
